; Einfaches NF-Testprogramm...
; verbindet AF_RX mit AUDO und AUDI mit AF_TX.
; Die Lautstrke und die Mikrofonverstrkung sind ber Cmd0/Cmd1 einstellbar.
; Min: $0, Erste hrbare Stufe: $10000, Max: $7FFFFF

START	equ	$0040

IPR	equ	$FFFF			; Priorittsregister Interrupts
BCR	equ	$FFFE			; Steuerregister BUS
PCTL	equ	$FFFD			; Steuerregister PLL
OGDBR	equ	$FFFC			; Register der OnCE-Schnittstelle

STXH	equ	$FFF6			; Sende- und Empfangsregister SCI High
SRXH	equ	$FFF6			; Sende- und Empfangsregister SCI High
STXM	equ	$FFF5			; Sende- und Empfangsregister SCI Mid
SRXM	equ	$FFF5			; Sende- und Empfangsregister SCI Mid
STX	equ	$FFF4			; Sende- und Empfangsregister SCI
SRX	equ	$FFF4			; Sende- und Empfangsregister SCI
STXA	equ	$FFF3			; Senderegister einer -Adresse fr SCI
SCCR	equ	$FFF2			; Steuerregister fr SCI-Takt
SSR	equ	$FFF1			; Statusregister fr SCI
SCR	equ	$FFF0			; Steuerregister SCI

SSIRX	equ	$FFEF			; Sende- / Empfangsregister SSI
SSITX	equ	$FFEF			; Sende- / Empfangsregister SSI
SSITSR	equ	$FFEE			; Temp-Fenster fr SSI
CRB	equ	$FFED			; Steuerregister fr SSI
CRA	equ	$FFEC			; Steuerregister fr SSI
HTX	equ	$FFEB			; Sende- / Empfangsregister HostIO
HRX	equ	$FFEB			; Sende- / Empfangsregister HostIO
HSR	equ	$FFE9			; Statusregister HostIO
HCR	equ	$FFE8			; Steuerregister HostIO

PCD	equ	$FFE5			; Datenregister Port C
PDB	equ	$FFE4			; Datenregister Port B
PCDDRC	equ	$FFE3			; Directionregister Port C
PCDDRB	equ	$FFE2			; Directionregister Port B
PCC	equ	$FFE1			; Controlregister Port C
PBC	equ	$FFE0			; Controlregister Port B
TCR	equ	$FFDF			; Zhlerregister des Zeitgebers/Zhlers


; Variablen X-Bereich
Volume		equ	$0		; Lautesprecher / Hrerlautstrke
MicGain		equ	$1		; Miktofonverstrkung / Hub
ToneHub		equ	$2		; Sinustonausgabe: Hub
ToneVolume	equ	$3		; Sinustonausgabe: Mithrlautstrke

TCtrl		equ	$4		; ToneControl: Bit0 schaltet Ton ein
NextSinOfs	equ	$5		; 
SinDelta	equ	$6
SinPos		equ	$7

; Variablen Y-Bereich
AUDI		equ	$0		; SampleWert vom Mikrofon -> HF-Teil
AUDO		equ	$1		; SampleWert vom HF-Teil -> Hrer


; Die folgenden Macros stammen aus einem Beispiel und wurden unverndert ber-
; nommen.

;
; mulin - mu-law to linear conversion (A1->A1)
;
mulin   macro
_shift	equ	$80                     ;shift constant
_mutable equ	$100			;base address of mu-law table
        move            a1,b            ;save input
        lsl     a       #>_shift,x0     ;shift out sign bit, get shift constant
        lsr     a       #_mutable,x1    ;shift in zero, get table base
        tfr     x1,a    a1,x1           ;swap table base and offset
        mac     x1,x0,a                 ;shift offset down and add to base
        move            a,r0            ;move to address register
        nop
        lsl     b       x:(r0),a        ;c=sign bit, lookup linear data
        neg     a       a,b             ;a=negative result, b=positive result
        tcs     b,a                     ;if pos sign, correct result
        endm
;
; linmu - linear to mu-law conversion (A1->A1)
;
linmu	macro
_bias	equ	$008400			;absolute bias = 33
	tfr	a,b	a,a		;save input sign, limit input data
	abs	a	#_bias,x0	;form input magnitude, get bias
	add	x0,a	#7,r0		;add bias to magnitude, get chord bar
	move		a,a		;limit again
	tst	a			;set CCR for norm instruction
	rep	#7			;find chord number by normalizing
	norm	r0,a			; biased magnitude to get step number
	asl	a			;isolate step number
	asl	a	b,b		;limit input again
	neg	b	r0,a2		;invert sign bit, get chord number
	asr	a			;combine chord and step
	asr	a
	asr	a
	not	a			;invert 7 LSB's for mu-law
	asl	b			;get sign bit
	ror	a	#<$ff,x0	;combine sign, chord and step
	and	x0,a			;clear 16 LSB's
	endm


;******************************************************************************
;* Start des Programmes
;******************************************************************************

	ORG     P:$0
	jmp     START			; Init
	
	org	P:$000C			; Adresse des SSI-RX-Ints
	jsr	<SSI_INT

	org	P:$0014			; Adresse des SCI-RX-Ints
	jsr	<SCI_INT
	
	
	org	P:$0024			; HI: SetVolume
	movep	X:HRX,X:Volume
	org	P:$0026			; HI: SetMicGain
	movep	X:HRX,X:MicGain
	org	P:$0028			; HI: SetToneHub (AusgabeHub Rufton)
	movep	X:HRX,X:ToneHub
	org	P:$002A			; HI: SetTonVolume (Mithrlautstrke)
	movep	X:HRX,X:ToneVolume
	org	P:$002C			; HI: Set ON/OFF-Flags!
	movep	X:HRX,X:TCtrl
	
	org	P:START
Init	Reset
	move	#$100,R0
	move	#$FF,M0
	movec	#$04,OMR		; DataROM enable, Mode0.

	
	move	#$0100,R1		; Sinustabelle ansprechen
	move	#$00FF,M1
	move	#47,N1

	movep	#$01,X:PBC		; Config PortB as HostInterface
	movep	#$04,X:HCR		; CommandIRQ enable
	
	movep	#$01FF,X:PCC		; Config PortC as SSI /SCI
	movep	#$B20C,X:CRB            ; Config SSI, Tx, RX, RXIRQ	
	movep	#$0B08,X:SCR		; SCI Config
	movep	#$C000,X:SCCR		; Takt fr SCI ist extern	
	movep	#$5400,X:IPR		; SCUI + SSI + HI auf Ebene 0
	

	move	#0.0,X0			; InitVolume
	move	#0.85,X1		; Init MicGain 
	move	X0,X:<Volume	
	move	X1,X:<MicGain
	move	#0.85,X0		; Init ToneHub
	move	#0.12,X1		; Init ToneVolume
	move	X0,X:<ToneHub
	move	X1,X:<ToneVolume
	
	bclr	#0,X:<TCtrl		; Test Tone einschalten
	
	move	#$2EAAAA,X0
	move	#0,X1
	move	X0,X:<SinDelta		; Delta fr Sinustabelle
	move	X1,X:<SinPos
	
	;***	Ende Initialisierung
	movec	#$0000,SR		; alle ints erlaubt
	
Mainlp	wait
	jmp	Mainlp


SSI_INT	movep	X:SSIRX,A1		; Sample vom Mikrofon
	jset	#0,X:<TCtrl,SSI_GenerateSin
	; *** Ein/Ausgabe der Samples
		mulin
	move	A,X1	
	move	Y:<AUDO,X0
	move	X:<Volume,Y0
	move	X:<MicGain,Y1
	jmp	SSI_IO
	; *** Ausgabe eines Sinus-Tones
SSI_GenerateSin
	;clr	A
	;move	X:<SinPos,A0
	;do	#16,SIN_GetTabOfs
	;lsr	A
SIN_GetTabOfs
	;move	A0,N1			; Neuer Offset fr die Sinustabelle
	; *** Neuen Offset berechnen
	;move	X:<SinPos,A0
	;move	X0,X:<SinDelta	
	;move	#0,X1
	;nop
	;add	X,A
	;move	A0,X:<SinPos
	; *** Wert aus der Sinustabelle holen
	move	Y:(R1)+N1,X0
	;move	Y:(R1+N1),X0
	move	X0,X1
	move	X:<ToneVolume,Y0
	move	X:<ToneHub,Y1

	; *** Berechnen und Ausgabe der Samples:
SSI_IO	mpy	Y1,X1,B
	mpy	Y0,X0,A		B,Y:<AUDI
		linmu
	movep	A1,X:SSITX
	rti



SCI_INT	movep	X:SRXH,A1		; Einlesen AF.
		mulin
	move	A,Y:<AUDO
	nop
	move	Y:<AUDI,A
		linmu
	movep	A1,X:STXH		; Ausgeben AF.
	rti
